﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace CoscomV4_Tutorial
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        System.Timers.Timer FailsafeTimer = new System.Timers.Timer();
        public static NumberFormatInfo ifo = new NumberFormatInfo();

        public MainWindow()
        {
            InitializeComponent();

            // add all available serial ports to combobox
            foreach (string s in SerialPort.GetPortNames())
                AvailablePorts.Items.Add(s);
            if (AvailablePorts.Items.Count > 0)
                AvailablePorts.SelectedIndex = 0;

            // Init failsafe timer
            FailsafeTimer.AutoReset = true;
            FailsafeTimer.Interval = 500; // reset failsafe all 500 ms (1 second is due time, so this has some buffer)
            FailsafeTimer.Elapsed += FailsafeTimer_Elapsed;

            // init number format parser. For this example we only use "." for decimal points regardless of operating system settings
            ifo.NumberDecimalSeparator = ".";
            ifo.NumberGroupSeparator = "";
        }


        private void FailsafeTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        { // reset failsafe 
            try
            {
                if (MyService !=null)
                {
                    MyService.ResetFailsafe();
                }
            }
            catch (Exception)
            {

            }
        }

        private void SetSpeed_Click(object sender, RoutedEventArgs e)
        {
            if (MyService != null)
            {
                try
                {
                    // set a new drive speed
                    MyService.SetSpeed(double.Parse(TargetSpeed.Text,ifo), 0.2);
                }
                catch (Exception er)
                {
                    MessageBox.Show("Error: " + er.Message);
                }
            }
        }

        private void SetElevation_Click(object sender, RoutedEventArgs e)
        {
            if (MyService != null)
            {
                try
                {
                    // set a new elevation value
                    MyService.SetElevation(double.Parse(TargetElevation.Text,ifo));
                }
                catch (Exception er)
                {
                    MessageBox.Show("Error: " + er.Message);
                }
            }
        }

        private void Connect_Click(object sender, RoutedEventArgs e)
        {
            Connect.IsEnabled = false;
            // Search the device in a new thread
            Thread t = new Thread(new ParameterizedThreadStart(ConnectInTread));
            t.Start(AvailablePorts.Text);

        }
        
        XDOP.schemas_coscom_org.device.MCU6Treadmill_1 MyDevice =null; // device instance
        XDOP.schemas_coscom_org.service.ISBasicControlService MyService=null; // service instance
        private void ConnectInTread(object args)
        {
            try
            {
                // Create a new protocol parameter instance. nothing to change here. Default values are ok
                XDOP.Core.ProtocolOptions op = new XDOP.Core.ProtocolOptions();              

               

                // Create a new serial connection parameter for the specified com port and coscom v4 baud rate 19200
                XDOP.Transport.SerialTransport serialTransport = new XDOP.Transport.SerialTransport(new XDOP.Transport.SerialTransportParameter((string)args, 19200));


                MyDevice = new XDOP.schemas_coscom_org.device.MCU6Treadmill_1();

                // connect to the device
                MyDevice.Connect(serialTransport, op);

                // iterate through all services and save a reference to the direct control service
                foreach (CodeGenerator.IServiceObjectV4 service in MyDevice.Services)
                {
                    if (service.ServiceType.Equals(XDOP.schemas_coscom_org.service.BasicControlService.CodeGeneratorServiceType))
                        MyService = (XDOP.schemas_coscom_org.service.ISBasicControlService)service;
                }

                string deviceType = "";
                string serial = "";
                string version = "";
                MyService.GetDeviceInformation(ref deviceType, ref serial, ref version);
                this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] { DeviceType, deviceType });
                this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] { Serial, serial});
                this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] { Firmware, version });

                // check device information if device type is either MCU5 or MCU6
                if (!deviceType.Contains("MCU5") && !deviceType.Contains("MCU6"))
                {
                    MessageBox.Show("Device found but was no MCU device");
                    MyDevice.DisposeDevice();
                    MyDevice = null;
                    MyService = null;
                    return;
                }


                MyService.ActualElevationChanged += MyService_ActualElevationChanged;
                MyService.ActualSpeedChanged += MyService_ActualSpeedChanged;
                MyService.ControlAllowedChanged += MyService_ControlAllowedChanged;
                MyService.SetEventMask("1111111111111111111111"); // enable all events for simplicity. 

                this.Dispatcher.Invoke(new SetControlEnabled(SetControlEnabledImplementation), new object[] { Connect, false });
                this.Dispatcher.Invoke(new SetControlEnabled(SetControlEnabledImplementation), new object[] { Disconnect, true });
                this.Dispatcher.Invoke(new SetControlEnabled(SetControlEnabledImplementation), new object[] { gbControl, true});
            }
            catch (Exception ex)
            {
                this.Dispatcher.Invoke(new SetControlEnabled(SetControlEnabledImplementation), new object[] { Connect, true });
                this.Dispatcher.Invoke(new SetControlEnabled(SetControlEnabledImplementation), new object[] { Disconnect, false });
                this.Dispatcher.Invoke(new SetControlEnabled(SetControlEnabledImplementation), new object[] { gbControl, false });
                MessageBox.Show("Error: " + ex.Message);
            }
        }


        private void Disconnect_Click(object sender, RoutedEventArgs e)
        {
            // Dispose the device and set service instance to null
            if (MyDevice != null)
            {
                MyDevice.DisposeDevice();
                MyDevice = null;
                MyService = null;
            }
            Connect.IsEnabled = true;
            gbControl.IsEnabled = false; 
            Disconnect.IsEnabled = false;
        }


        private void MyService_ControlAllowedChanged(byte newValue)
        { // 0 = allowed, 1 = pending, 2 = not allowed
            if (newValue == 0 && !FailsafeTimer.Enabled)
            { // if allowed and failsafe timer not already running, enable it otherwise allowance will be removed after 1 second.
                FailsafeTimer.Start();
                this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] {FailsafeActive,"Yes" });
            }
            else
            { // if not allowed, stop failsafe timer
                FailsafeTimer.Stop();
                this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] { FailsafeActive, "No" });
            }
            this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] { ControlAllowed, newValue == 0 ?"Allowed" : (newValue == 1 ? "pending" : "not allowed") });
        }

        private void MyService_ActualSpeedChanged(double newValue)
        { // show new actual speed (m/s²)
            this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] { ActualSpeed, newValue.ToString("N2") });
        }

        private void MyService_ActualElevationChanged(double newValue)
        { // show new actual elevation value (%)
            this.Dispatcher.Invoke(new SetControlText(DelegateImplementation), new object[] { ActualElevation, newValue.ToString("N2") });
        }

        // delegate for setting GUI Textbox
        private delegate void SetControlText(TextBox c, string text);
        private void DelegateImplementation(TextBox c, string text)
        {
            c.Text = text;
        }

        // delegate for setting GUI Textbox
        private delegate void SetControlEnabled(Control b, bool isEnabled);
        private void SetControlEnabledImplementation(Control b, bool isEnabled)
        {
            b.IsEnabled = isEnabled;
        }

        private void RequestControl_Click(object sender, RoutedEventArgs e)
        {
            try
            {                
                if (MyService != null)
                    MyService.RequestControl(RequestControlMessage.Text); // request control with message written in textbox. 45 Characters max. MCU5 will not be able to display the message and ignore it.
            }
            catch (Exception er) 
            {
                MessageBox.Show("Error: " + er.Message);
            }
        }
    }
}
